--- Guilder is an addon written to attempt to make everyday guild life simpler and more enjoyable
-- The addon contains modules for guild admin as well as party/raid utilities
-- @class file
-- @name Guilder.lua
-- @release 2.0.x

--[[
   Copyright 2009 Christo Kotze (onaforeignshore)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
]]

-- localisation
local L = LibStub("AceLocale-3.0"):GetLocale("Guilder")

-- default settings for the addon -------------------------------------------

local DATABASE_DEFAULTS = {
	profile = {
		info = {
			showOnline = true,
			showTotal = true,
			showOffline = true,
			showModules = true,
			showHint = true
		},
		dbCache = {},
		minimap_icon = {
			hide = false,
			minimapPos = 200,
			radius = 80
		},
		comm = {
			isGuildMaster = false
		},
		showDebug = true
	}
}

-- localise global variables for faster access ------------------------------

local _G = _G
local date = _G.date
local ipairs = _G.ipairs
local pairs = _G.pairs
local table = _G.table
local type = _G.type
local wipe = _G.wipe

local IsInGuild = _G.IsInGuild
local IsInInstance = _G.IsInInstance
local UnitClass = _G.UnitClass
local UnitName = _G.UnitName

--[[ colours stored for code readability - used as a handy reference only
local RESET = "|r"
local BLIZZARD = "|cff00c0ff"
local BLUE = "|cff0070dd"
local CYAN = "|cff00c0ff"
local GREEN = "|cff1eff00"
local GREY = "|cff999999"
local ORANGE = "|cffff8400"
local PURPLE = "|cff9f3fff"
local RED = "|cffff0000"
local WHITE = "|cffffffff"
local YELLOW = "|cffffff00"
]]

-- declare the addon --------------------------------------------------------

Guilder = LibStub("AceAddon-3.0"):NewAddon("Guilder", "AceComm-3.0", "AceEvent-3.0", "AceHook-3.0", "AceSerializer-3.0", "LibSink-2.0")

-- put the addon in the global table
_G.Guilder = Guilder

-- enable debugging (for Development version)
Guilder.DEBUG = _G.Guilder_DEBUG or false
_G.Guilder_DEBUG = nil
local DEBUG = Guilder.DEBUG
Guilder.expect = _G.Guilder_expect
_G.Guilder_expect = nil
local expect = Guilder.expect

--[===[@debug@
--load tekDebug
local debugf = tekDebug and tekDebug:GetFrame("Guilder")
local function Debug(...) if debugf then debugf:AddMessage("["..date("%X").."] "..string.join(" ", tostringall(...))) end end
--@end-debug@]===]

-- get version from .toc - set to Development if no version
Guilder.version = GetAddOnMetadata("Guilder", "Version")
if Guilder.version:match("@") then
	Guilder.version = "Development"
end
-- add localisation to addon
Guilder.L = L
-- declare the database
local db

-- load additional libraries ------------------------------------------------

-- LibDataBroker used for LDB enabled addons like 
local LibDataBroker = LibStub("LibDataBroker-1.1", true)
if not LibDataBroker then
	LoadAddOn("LibDataBroker-1.1")
	local LibDataBroker = LibStub("LibDataBroker-1.1", true)
	if not LibDataBroker then
		error(format(L["%s requires the library '%s' to be available."], L["Guilder"], "LibDataBroker-1.1"))
	end
end
-- LibGuild creates a lookup table for quick multiple lookups
LibGuild = LibStub("LibGuild-1.0", true)
if not LibGuild then
	LoadAddOn("LibGuild-1.0")
	LibGuild = LibStub("LibGuild-1.0", true)
	if not LibGuild then
		error(format(L["%s requires the library '%s' to be available."], L["Guilder"], "LibGuild-1.0"))
	end
end
-- LibGroup creates a lookup table for quick multiple lookups for a raid or party
LibGroup = LibStub("LibGroup-1.0", true)
if not LibGroup then
	LoadAddOn("LibGroup-1.0")
	LibGroup = LibStub("LibGroup-1.0", true)
	if not LibGroup then
		error(format(L["%s requires the library '%s' to be available."], L["Guilder"], "LibGroup-1.0"))
	end
end

-- global variables ---------------------------------------------------------

-- store name, faction and realm for quick access
myName = UnitName("player")
myFaction, _ = UnitFactionGroup("player")
myRealm = GetRealmName()
myNameRealm = myName.." - "..myRealm
_, myClass = UnitClass("player")
if not myClass then
	myColorName = "|cffcccccc"..myName.."|r"
else
	local c = RAID_CLASS_COLORS[myClass]
	myColorName = "|cff"..("%02X%02X%02X"):format(c.r * 255, c.g * 255, c.b * 255)..myName.."|r"
end
--check when configuration is loaded before changing it
Guilder.configLoaded = false

-- local variables ----------------------------------------------------------

-- don't update rank tables on config screen unless required
local configUpdated = false
-- definte a communications prefix
local commPrefix = "GLDR"
-- table of Guilder users in the guild
local guilderUsers = {}
-- boss mod versions
local bossMods = {}

-- create table cache -------------------------------------------------------

do
	-- unused tables go in this set
	-- if the garbage collector comes around, they'll be collected properly
	local cache = setmetatable({}, {__mode='k'})
	
	--- Return a table
	-- @usage local t = Guilder.new()
	-- @return a blank table
	function Guilder.new()
		local t = next(cache)
		if t then
			cache[t] = nil
			return t
		end
		
		return {}
	end
	
	--- Delete a table, clearing it and putting it back into the queue
	-- @usage local t = Guilder.new()
	-- t = del(t)
	-- @return nil
	function Guilder.del(t)
		--[===[@debug@
		if DEBUG then
			expect(t, 'typeof', 'table')
			expect(t, 'not_inset', cache)
		end
		--@end-debug@]===]
		wipe(t)
		cache[t] = true
		return nil
	end
end

--- Wrap the given function so that any call to it will be piped through Guilder:RunOnLeaveCombat.
-- @param func function to call
-- @usage myFunc = Guilder:OutOfCombatWrapper(func)
-- @usage MyNamespace.MyMethod = Guilder:OutOfCombatWrapper(MyNamespace.MyMethod)
-- @return the wrapped function
function Guilder:OutOfCombatWrapper(func)
	--[===[@debug@
	if DEBUG then
		expect(func, 'typeof', 'function')
	end
	--@end-debug@]===]
	return function(...)
		return Guilder:RunOnLeaveCombat(func, ...)
	end
end

-- modules ------------------------------------------------------------------

--- Call a given method on all modules if those modules have the method. This will iterate over disabled modules.
-- @param method_name name of the method
-- @param ... arguments that will be passed to the module
function Guilder:CallMethodOnModules(method_name, ...)
	for id, module in self:IterateModules() do
		if module[method_name] then
			module[method_name](module, ...)
		end
	end
end

-- Standard Methods ---------------------------------------------------------

-- The first function called when the addon loads
-- @usage internal use only
function Guilder:OnInitialize()
	db = LibStub("AceDB-3.0"):New("GuilderDB", DATABASE_DEFAULTS, "Default")
	DATABASE_DEFAULTS = nil
	self.db = db
	
	-- make sure we update if the profile is changed
	db.RegisterCallback(self, "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	
	-- used for run-once-only initialisation
	self:AddonLoaded()
	
	-- check if we're the guild master of the guild
	if IsInGuild() and LibGuild:GetGuildLeader() == myName then self.db.profile.comm.isGuildMaster = true end
end

-- check if LDB icon is loaded
local db_icon_done

-- sort table pairs alphabetically by key
-- @param t table to be sorted
-- @param f sort function to be used (can be omitted)
-- @usage internal use only
--     for k, v in pairsByKeys(someTable) do
--         print(k.." = ".. v)
--     end
local function pairsByKeys (t, f)
	local a = {}
		for n in pairs(t) do table.insert(a, n) end
		table.sort(a, f)
		local i = 0      -- iterator variable
		local iter = function ()   -- iterator function
			i = i + 1
			if a[i] == nil then
				return nil
			else
				return a[i], t[a[i]]
			end
		end
	return iter
end

-- Used for run-once-only initialisation
-- @usage internal use only
function Guilder:AddonLoaded()
	-- create LibDataBroker Launcher
	if not Guilder.LibDataBrokerLauncher then
		if LibDataBroker then
			Guilder.LibDataBrokerLauncher = LibDataBroker:NewDataObject("Guilder", {
				type = "launcher",
				icon = [[Interface\AddOns\Guilder\Guilder]],
				OnClick = function(clickedframe, button)
					if button == "LeftButton" then
						return Guilder.Options.OpenConfig()
					elseif button == "RightButton" then
						--[[
						Guilder.db.profile.modules["Busy Responder"] = not Guilder.db.profile.modules["Busy Responder"]
						if Guilder.db.profile.modules["Busy Responder"] then
							Guilder:EnableModule("Busy Responder")
							Guilder:Print(L["Enabled"], "Busy Responder", L["Module"])
						else
							Guilder:DisableModule("Busy Responder")
							Guilder:Print(L["Disabled"], "Busy Responder", L["Module"])
						end
						-- LibStub("AceConfigRegistry-3.0"):NotifyChange("Guilder")
						]]
					end
				end,
				OnTooltipShow = function(tt)
					tt:AddLine(L["Guilder"])
					tt:AddLine(" ")
					if IsInGuild() then
						tt:AddLine("|cff00c0ff"..L["Officers"].."|r")
						for name in LibGuild:GetIterator("NAME", true) do
							if Guilder:IsOfficer(name) then
								oAlt = Guilder:FindOfficerAlt(name)
								if LibGuild:IsMemberOnline(name) then
									tt:AddLine("|cff1eff00"..name.."|r")
								elseif oAlt ~= nil then
									tt:AddLine("|cffffff00"..oAlt.."|r")
								elseif Guilder.db.profile.info.showOffline then
									tt:AddLine("|cffff0000"..name.."|r")
								end
							end
						end
					else
						tt:AddLine(L["You are not in a guild!"])
					end
					if Guilder.db.profile.info.showModules then
						tt:AddLine(" ")
						tt:AddLine("|cff00c0ff"..L["Modules"].."|r")
						local status = {}
						for id, module in Guilder:IterateModules() do
							status[module.name] = module.db.profile.enabled
						end
						for k, v in pairsByKeys(status) do
							tt:AddLine("|cffffffff"..k.."|r "..
								(v and "|cff1eff00"..L["Enabled"].."|r" or "|cffff0000"..L["Disabled"].."|r"))
						end
					end
					if Guilder.db.profile.info.showHint then
						tt:AddLine(" ")
						tt:AddLine("|cff1eff00"..L["Hint"].."|r")
						tt:AddLine(L["%s to open the options menu"]:format("|cffffff00"..L["Click"].."|r"), 1, 1, 1)
						--tt:AddLine(L["%s to toggle Busy Responder"]:format("|cffffff00"..L["Right-click"].."|r"), 1, 1, 1)
					end
				end,
			})
		end
	end
	-- create the Launcher icon
	if not db_icon_done and Guilder.LibDataBrokerLauncher then
		local LibDBIcon = LibStub("LibDBIcon-1.0", true)
		if LibDBIcon and not IsAddOnLoaded("Broker2FuBar") then
			LibDBIcon:Register("Guilder", Guilder.LibDataBrokerLauncher, Guilder.db.profile.minimap_icon)
			db_icon_done = true
		end
	end
	-- link LibSink to the db
	self:SetSinkStorage(self.db.profile)
	-- first time this character is using the addon, so create and initialise rank tables
	if not self.db.profile.dbCache[myNameRealm] or not self.db.profile.dbCache[myNameRealm].RankNames then
		self.db.profile.dbCache[myNameRealm] = {}
		-- initialise dbCache
		self.db.profile.dbCache[myNameRealm].RankNames = {}
		self.db.profile.dbCache[myNameRealm].ranks = {}
		for i = 1, 10 do
			self.db.profile.dbCache[myNameRealm].RankNames[i] = "Rank "..i
			self.db.profile.dbCache[myNameRealm].ranks[i] = "3-Member"
		end
	end
	-- add my version to the table
	guilderUsers[myName] = Guilder.version
	-- announce we are using Guilder
	self:SendComm("MAIN", "GUILD", myName, "UGVA", Guilder.version)
	-- request version information from other guild members
	self:SendComm("MAIN", "GUILD", myName, "QGVA")
end

do
	local function find_Guilder(...)
		for i = 1, select('#', ...) do
			if (select(i, ...)) == "Guilder" then
				return true
			end
		end
		return false
	end
	
	local function iter(num_addons, i)
		i = i + 1
		if i >= num_addons then
			-- and we're done
			return nil
		end
		
		-- must be Load-on-demand (obviously)
		if not IsAddOnLoadOnDemand(i) then
			return iter(num_addons, i)
		end
		
		local name = GetAddOnInfo(i)
		
		-- must start with Guilder_
		local module_name = name:match("^Guilder_(.*)$")
		if not module_name then
			return iter(num_addons, i)
		end
		
		-- Guilder must be in the Dependency list
		if not find_Guilder(GetAddOnDependencies(i)) then
			return iter(num_addons, i)
		end
		
		local condition = GetAddOnMetadata(name, "X-Guilder-Condition")
		if condition then
			local func, err = loadstring(condition)
			if func then
				-- function created successfully
				local success, ret = pcall(func)
				if success then
					-- function called and returned successfully
					if not ret then
						-- shouldn't load, e.g. DruidManaBar when you're not a druid
						return iter(num_addons, i)
					end
				end
			end
		end
		
		-- passes all tests
		return i, name, module_name
	end
	
	--- Return a iterator of addon ID, addon name that are modules that Guilder can load.
	-- module_name is the same as name without the "Guilder_" prefix.
	-- @usage for i, name, module_name in Guilder:IterateLoadOnDemandModules() do
	--     print(i, name, module_name)
	-- end
	-- @return an iterator which returns id, name, module_name
	function Guilder:IterateLoadOnDemandModules()
		return iter, GetNumAddOns(), 0
	end
end

-- list of the modules that are not loaded
local modules_not_loaded = {}
Guilder.modules_not_loaded = modules_not_loaded

--- Load Load-on-demand modules if they are enabled and exist.
-- @usage Guilder:LoadModules()
function Guilder:LoadModules()
	-- NOTE: this assumes that module profiles are the same as Guilder's profile.
	local current_profile = self.db:GetCurrentProfile()
	
	local sv = self.db.sv
	local sv_namespaces = sv and sv.namespaces
	for i, name, module_name in self:IterateLoadOnDemandModules() do
		local module_sv = sv_namespaces and sv_namespaces[module_name]
		local module_profile_db = module_sv and module_sv.profiles and module_sv.profiles[current_profile]
		local enabled = module_profile_db and module_profile_db.enabled
		
		if enabled == nil then
			-- we have to figure out the default state
			local default_state = GetAddOnMetadata(name, "X-Guilder-DefaultState")
			enabled = (default_state ~= "disabled")
		end

		local loaded
		if enabled then
			--[===[@debug@
			if self.db.profile.showDebug then
				self:ShowMessage(("Found module '%s', attempting to load."):format(module_name))
			end
			--@end-debug@]===]
			loaded = LoadAddOn(name)
		end
	
		if not loaded then
			--[===[@debug@
			if self.db.profile.showDebug then
				self:ShowMessage(("Found module '%s', not loaded."):format(module_name))
			end
			--@end-debug@]===]
			modules_not_loaded[module_name] = true
		end
	end
end

-- Load the module with the given id and enable it
-- @param id id of the module
-- @usage internal usage only
--     Guilder:LoadAndEnableModule()
-- @returns nil
function Guilder:LoadAndEnableModule(id)
	local loaded, reason = LoadAddOn('Guilder_' .. id)
	if loaded then
		local module = self:GetModule(id)
		assert(module)
		self:EnableModule(module)
	else
		if reason then
			reason = _G["ADDON_"..reason]
		end
		if not reason then
			reason = UNKNOWN
		end
		self:ShowMessage(format(L["%s: Could not load module '%s': %s"], "Guilder", id, reason))
	end
end

-- Called when the user changes his/her profile settings.
-- @usage internal use only
function Guilder:OnProfileChanged()
	-- Notify modules that the profile has changed.
	for _, module in Guilder:IterateEnabledModules() do
		if module.OnProfileChanged then
			module:OnProfileChanged()
		end
	end
	-- localise the db
	local db = self.db
	-- load the modules again
	self:LoadModules()
	-- Enable/Disable modules to match the new profile.
	for _, module in self:IterateModules() do
		if module.db.profile.enabled then
			self:EnableModule(module)
		else
			self:DisableModule(module)
		end
	end
	-- if we're not using FuBar with the Broker2FuBar library, show the minimap icon
	if not IsAddOnLoaded("Broker2FuBar") then
		local libdbicon = LibStub("LibDBIcon-1.0")
		local minimap_icon_db = db.profile.minimap_icon
		libdbicon:Refresh("Guilder", minimap_icon_db)
		if minimap_icon_db.hide then
			libdbicon:Hide("Guilder")
		else
			libdbicon:Show("Guilder")
		end
	end
end

local timerFrame = CreateFrame("Frame")
timerFrame:Hide()

-- Called when the addon is enabled.
-- @usage internal use only
function Guilder:OnEnable()
	-- register zone change events
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	-- enter/leave combat for :RunOnLeaveCombat
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	-- register guild
	if IsInGuild() then
		-- if we are in a guild, register a callback to update the LDB text
		LibGuild:RegisterCallback("Update", function() Guilder:UpdateText() end)
	end
	-- Update the ranks if the player leaves / joins a guild
	self:RegisterEvent("PLAYER_GUILD_UPDATE")
	-- register entering the game
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	
	timerFrame:Show()
	-- do initial update
	self:OnProfileChanged()
	-- register Comm framework
	self:RegisterComm(commPrefix)
	-- listen in on other addons
	self:RegisterEvent("CHAT_MSG_ADDON")
	-- if we're the GM, tell the users to request an update the moment we come online
	if LibGuild:GetGuildLeader() == myName then
		self:SendComm("MAIN", "GUILD", myName, "UGRT")
	else -- otherwise request an update
		self:SendComm("MAIN", "GUILD", myName, "QGRT")
	end
end

-- Called when the addon is disabled
-- @usage internal use only
function Guilder:OnDisable()
	self:UnregisterAllEvents()
	LibGuild:UnregisterAllEvents()	
end

-- Called when a player enters a new zone
-- @usage internal use only
function Guilder:ZONE_CHANGED_NEW_AREA()
end

-- Called when a player first becomes active in the gaming world
-- @usage internal use only
function Guilder:PLAYER_ENTERING_WORLD()
end

-- Updates the ranks if the player joins or leaves a guild
-- @usage internal use only
function Guilder:PLAYER_GUILD_UPDATE()
	if IsInGuild() then
		-- we joined a guild, get the ranks
		if configLoaded then self:UpdateConfigOptions() end
	else
		-- we left a guild, reset the cached ranks
		for i = 1, 10 do
			self.db.profile.dbCache[myNameRealm].RankNames[i] = "Rank "..i
			self.db.profile.dbCache[myNameRealm].ranks[i] = "3-Member"
		end
	end
end

do
	local in_combat = false
	Guilder.in_combat = in_combat
	local actions_to_perform = {}
	local pool = setmetatable({}, {__mode='k'})
	
	-- called when we leave combat
	function Guilder:PLAYER_REGEN_ENABLED()
		in_combat = false
		for i, t in ipairs(actions_to_perform) do
			t.f(unpack(t, 1, t.n))
			actions_to_perform[i] = nil
			wipe(t)
			pool[t] = true
		end
	end
	
	-- called when we enter combat
	function Guilder:PLAYER_REGEN_DISABLED()
		-- Important Note: Since the release of patch 3.2, certain fights in the game fire the 
		-- "PLAYER_REGEN_DISABLED" event continuously during combat causing any subsequent events
		-- we might trigger as a result to also fire continuously. It is recommended therefore to
		-- use a checking variable that is set to 'on/1/etc' when entering combat and back to
		-- 'off/0/etc' only when exiting combat and then use this as the final determinant on
		-- whether or not to action a subsequent event.
		in_combat = true
	end
	
	--- Call a function if out of combat or schedule to run once combat ends.
	-- If current out of combat, the function provided will be called without delay.
	-- @param func function to call
	-- @param ... arguments to pass into func
	-- @usage Guilder:RunOnLeaveCombat(someSecureFunction)
	-- @usage Guilder:RunOnLeaveCombat(someSecureFunction, "player")
	function Guilder:RunOnLeaveCombat(func, ...)
		--[===[@debug@
		if DEBUG then
			expect(func, 'typeof', 'function')
		end
		--@end-debug@]===]
		if not in_combat then
			-- out of combat, call right away and return
			func(...)
			return
		end
		in_lockdown = InCombatLockdown() -- still in PLAYER_REGEN_DISABLED
		if not in_lockdown then
			func(...)
			return
		end
		-- get a table from the pool
		local t = next(pool) or {}
		pool[t] = nil
		-- save the function and parameters to the table
		t.f = func
		local n = select('#', ...)
		t.n = n
		for i = 1, n do
			t[i] = select(i, ...)
		end
		-- add the table to the out of combat actions_to_perform list
		actions_to_perform[#actions_to_perform+1] = t
	end
end

--- Updates the DataBroker text
-- @param nil
-- @usage Guilder:UpdateText
-- @returns nil
function Guilder:UpdateText()
	-- if we're in a guild
	if IsInGuild() then
		-- create the text to display
		if Guilder.db.profile.info.showOnline or Guilder.db.profile.info.showTotal then
			local temptext = ""
			local needslash = false
			-- we want to see how many are online
			if Guilder.db.profile.info.showOnline then
				temptext = temptext..self:GetNumberOfGuildOfficersOnline()
				needslash = true
			end
			-- we want to see how many in total
			if Guilder.db.profile.info.showTotal then
				if needslash then temptext = temptext.."/" end
				temptext = temptext..self:GetNumberOfGuildOfficers()
			end
			-- if we want to see online and total, use "Online"
			if Guilder.db.profile.info.showOnline then
				temptext = temptext.." "..L["Online"]
			else --if we only want to see total, use "Officers"
				temptext = temptext.." "..L["Officers"]
			end
			-- add the text to the Data Broker
			Guilder.LibDataBrokerLauncher.text = temptext
		else
			-- we're don't want to see officer stats, so show the addon name
			Guilder.LibDataBrokerLauncher.text = L["Guilder"]
		end
	else -- if we're not in a guild
		-- if we ask to see officer stats, show "No Guild"
		if Guilder.db.profile.info.showOnline or Guilder.db.profile.info.showTotal then
			Guilder.LibDataBrokerLauncher.text = L["No Guild"]
		else -- otherwise show the addon name
			Guilder.LibDataBrokerLauncher.text = L["Guilder"]
		end
	end
end

--- Is used to dynamically update the options in the configuration screen.
-- Modules may define their own MyModule:UpdateConfigOptions() to do the same.
-- This method is automatically called if it exists prior to the user opening the configuration screen.
-- @usage internal use only
-- @return nil
function Guilder:UpdateConfigOptions()
	-- check that we're in a guild
	if IsInGuild() then
		-- don't do anything if the options haven't been created for the first time yet
		if not configUpdated and Guilder.configLoaded then
			local r = {}
			-- get rank names
			for i = 1, GuildControlGetNumRanks() do
				local rankName = GuildControlGetRankName(i)
				-- check if the cached copy matches, otherwise update it
				if rankName ~= self.db.profile.dbCache[myNameRealm].RankNames[i] then
					self.db.profile.dbCache[myNameRealm].RankNames[i] = rankName
				end
				-- create the option if it doesn't exist or doesn't match
				if not r["rank"..i] or r["rank"..i].name ~= rankName then
					r["rank"..i] = {
					  	type = "select",
					  	name = rankName,
						desc = L["Select the rank type for this rank"],
						order = i,
						disabled = LibGuild:GetGuildLeader() ~= myName,
						values = { ["1-Officer"] = L["Officer"], ["2-OffAlt"] = L["Officer Alt"], ["3-Member"] = L["Member"], 	["4-Alt"] = L["Alt"] },
						get = function() return self.db.profile.dbCache[myNameRealm].ranks[i] end,
						set = function(info, v)
							self.db.profile.dbCache[myNameRealm].ranks[i] = v
							self:UpdateText()
						end
					}
				end
			end
			-- add the disclaimer
			r.disclaimer = {
				type = "description",
				name = L["Please note that the rank settings can only be changed by the Guild Master, however, they still appear here for your information."],
				order = 0
			}
			-- place rank names in the config screen
			Guilder.options.args.defaultArgs.args.ranks.args = r
			-- add an update button for the GM
			if LibGuild:GetGuildLeader() == myName then
				Guilder.options.args.defaultArgs.args.ranks.args.sendUpdate = {
					type = "execute",
					name = L["Send Update"],
					desc = L["If you have changed the ranks, notify the rest of the guild by clicking on this button"],
					order = 11,
					func = function() self:SendComm("MAIN", "GUILD", myName, "UGRT") end
				}
			else -- or a request update button for everyone else
				Guilder.options.args.defaultArgs.args.ranks.args.requestUpdate = {
					type = "execute",
					name = L["Request Update"],
					desc = L["If you think that the ranks are incorrect, you can request an update by clicking on this button"],
					order = 11,
					func = function() self:SendComm("MAIN", "GUILD", myName, "QGRT") end
				}
			end
			--[===[@debug@
			Guilder.options.args.defaultArgs.args.general.args.h1 = {
				type = "header",
				name = "",
				order = 7
			}
			Guilder.options.args.defaultArgs.args.general.args.users = {
				type = "execute",
				name = "User List",
				order = 8,
				func = function()
					for k, v in pairsByKeys(guilderUsers) do
						self:ShowMessage(k.." - "..v)
					end
				end
			}
			Guilder.options.args.defaultArgs.args.general.args.window = {
				type = "execute",
				name = "Test Window",
				order = 9,
				func = function()
					
				end
			}
			--@end-debug@]===]
			-- don't do this again
			configUpdated = true
		end
	else
		-- user not in a guild so display a message instead
		Guilder.options.args.defaultArgs.args.ranks.args = {
			info = {
				type = "description",
				name = L["You are not currently in a guild!"]
			}
		}
	end
end

-- addon communication functions --------------------------------------------

--- Is used to send information between users of the addon
-- @param mod the id of the module sending the message
-- @param distribution which channel to use. Valid options are "GUILD", "PARTY", "RAID" and "WHISPER"
-- @param target if distribution is a "WHISPER", this is the person to send the info to.
-- @param ... the content of the message
-- @usage Guilder:SendComm(self.id, "GUILD", nil, "some", "information")
-- @returns nil
function Guilder:SendComm(mod, distribution, target, ...)
	--[===[@debug@
	if DEBUG then
		expect(mod, 'typeof', 'string')
		expect(distribution, 'typeof', 'string')
		expect(target, 'typeof', 'nil;string')
	end
	--@end-debug@]===]
	-- make sure we don't use 'nil' for the target
	target = target or ""
	-- check that the distributions are valid or exit
	if (distribution == "WHISPER" and (not target or target == "")) or
		(distribution == "GUILD" and not IsInGuild()) or
		(distribution == "PARTY" and not Guilder:IsInParty()) or
		(distribution == "RAID" and not Guilder:IsInRaid()) then return end
	-- serialise the module and the parameters
	local msg = self:Serialize(mod, ...)
	-- send the message
	self:SendCommMessage(commPrefix, msg, distribution, target)
end

-- Local function to send the message to the right CommHandler() function.
-- Each module that wants to make use of the communications channel has to define its own MyModule:CommHandler()
-- to handle the message.
-- @param distribution which channel was used
-- @param sender who sent the message
-- @param result result of the Deserialize command
-- @param module the id of the module that sent the message
-- @param ... the content of the message
-- @usage internal use only
local function DispatchComms(distribution, sender, result, module, ...)
	if not result then -- couldn't deserialize the message
		if Guilder.db.profile.showDebug then
			Guilder:ShowMessage("Could not deserialize the message - "..module)
		end
		return
	end
	-- this is a message from the framework
	if module == "MAIN" then
		-- forward the message to the framework's CommHandler()
		Guilder:CommHandler(distribution, sender, ...)
	else -- this is a message from a module
		-- check that the CommHandler exists and is a function
		if type(Guilder.modules[module].CommHandler) ~= "function" then
			--[===[@debug@
			if self.db.profile.showDebug then
				Guilder:ShowMessage("No CommHandler found for "..module)
			end
			--@end-debug@]===]
			return
		end
		-- forward the message to the module's CommHandler()
		Guilder.modules[module]:CommHandler(distribution, sender, ...)
	end
end

-- Listens for addon communication and forwards messages for Guilder to the right place
-- @param prefix prefix of the comm message
-- @param message the message that was sent
-- @param distribution the channel that was used
-- @param sender the person that sent the message
-- @usage internal use only
function Guilder:OnCommReceived(prefix, message, distribution, sender)
	-- check if the comm message is meant for us
	if prefix == commPrefix then
		-- send it to the dispatcher
		DispatchComms(distribution, sender, self:Deserialize(message))
	end
end

-- Listens to other addon communications and stores information required
-- @param event not used
-- @param prefix prefix of the comm message
-- @param message the message that was sent
-- @param distribution the channel that was used
-- @param sender the person that sent the message
function Guilder:CHAT_MSG_ADDON(event, prefix, message, distribution, sender)
	--[===[@debug@
	-- clear out the spam addons
	if event ~= "CHAT_MSG_ADDON" -- event should always be "CHAT_MSG_ADDON"
	or prefix == "WIM" -- WIM
	or prefix == "DBMv4-Mod" or prefix == "DBMv4-Pull" -- DBM v4 events we don't want to see
	or prefix == "CTRA" --
	or prefix == "PLPWR" -- Pally Power
	or prefix == "QHpr" -- Quest Helper
	or prefix == "HealComm" -- HealComm
	or prefix == "LHC40" -- ??
	or prefix == "GSY" -- Gear Score
	or prefix == "GathX" -- Gatherer
	or prefix == "LGP" -- Looking for Group?
	or prefix == "LibGroupTalents-1.0" -- Lib Group Talents
	or prefix == "Altoholic" -- Altoholic
	or prefix == "DataStore" or prefix == "DS_Cont" or prefix == "DS_Craft" or prefix == "DS_Inv" -- Datastore
	or prefix == "Quixote2" -- Quixote 2 library
	or prefix == "SimpleMD" -- Simple Misdirection
	or prefix == "ForteWarlock" -- Forte Warlock
	or prefix == "HealBot" or prefix == "hbComms" -- Heal Bot
	or prefix == "BugSack?" -- BugSack error sent to user
	or prefix == "X-Perl" -- X-Perl Unit Frames
	or prefix == "RECOUNT" -- Recount
	or prefix == "LOOT_OPENED" then -- don't care about loot
		return
	end
	Debug("|cff9999ff"..distribution.."|r", "["..sender.."]", "|cffff00ff"..prefix.."|r ", message)
	--@end-debug@]===]
	-- flags to test for addons, so that we don't test for a lower revision if
	-- a higher revision is found. May feel counter-intuitive, but setting to
	-- false when found means we can test without using the 'not' operator.
	local BigWigs, DBM, DXE = true, true, true
	-- listen in on BigWigs 3 version events
	-- revision
	if prefix == "BWVR3" or prefix == "BWVQ3" then
		if not bossMods[sender] then bossMods[sender] = {} end
		bossMods[sender].BW = { addon = "BigWigs", rev = message, ver = "3.x" }
		BigWigs = false
	end
	-- listen in on Deadly Boss Mods 4 version events
	-- revision \t version \t version \t locale
	if prefix == "DBMv4-Ver" and message ~= "Hi!" then
		local args = self:GetArgs(message, "\t")
		if not bossMods[sender] then bossMods[sender] = {} end
		bossMods[sender].DBM = { addon = "DBM", rev = args[1], ver = args[2] }
		DBM = false
	end
	-- listen in on Deus Vox Encounters version events
	--
	
	-- listen in on BigWigs 2 version events
	-- module:revision
	if BigWigs and prefix == "BWVB2" then
		local module, revision = select(3, message:find("(.*):(%d+)"))
		if not module or not revision then return end
		if not bossMods[sender] then bossMods[sender] = {} end
		bossMods[sender].BW = { addon = "BigWigs", rev = revision, ver = "2.x" }
	end
	-- listen in on BigWigs 1 version events
	-- revision
	if BigWigs and prefix == "BWOOD" then
		if not bossMods[sender] then bossMods[sender] = {} end
		bossMods[sender].BW = { addon = "BigWigs", rev = message, ver = "1.x" }
	end
	-- listen in on Deadly Boss Mods 3 version events
	--
	
	-- listen in on Deadly Boss Mods 2 version events
	--
	
	-- listen in on Deadly Boss Mods 1 version events
	--
	
end

--- Is the framework's default CommHandler. It should NEVER be called from a module.
-- Instead, define a MyModule:CommHandler() to handle the module's own messages.
-- The framework will automatically forward the message to the module if this method is found.
-- @param distribution the channel that was used
-- @param sender the person that sent the message
-- @param command the command that was sent to the CommHandler
-- @param ... the message that goes with the command (if any)
-- @usage internal use only
function Guilder:CommHandler(distribution, sender, command, ...)
	-- only handle comms if they are not from yourself
	if sender == myName then return end
	local t = {}
	-- number of parameters
	local n = select('#', ...)
	-- Query: Guilder Version Announcement
	if command == "QGVA" then
		self:SendComm("MAIN", "WHISPER", sender, "RGVA", Guilder.version)
	end
	-- Response: Guilder Version Announcement
	if command == "RGVA" then
		if n ~= 1 then
			--[===[@debug@
			if self.db.profile.showDebug then
				self:ShowMessage("RGVA had no parameters!")
			end
			--@end-debug@]===]
			return
		end
		guilderUsers[sender] = select(1, ...)
	end
	-- Update: Guilder Version Announcement
	if command == "UGVA" then
		-- check that there is a parameter
		if n ~= 1 then
			--[===[@debug@
			if self.db.profile.showDebug then
				self:ShowMessage("UGVA had no parameters!")
			end
			--@end-debug@]===]
			return
		end
		-- update the guilder users table
		guilderUsers[sender] = select(1, ...)
	end
	-- Query: Get Rank Table - someone asked for a rank table update
	if command == "QGRT" then
		-- only respond to this command if we're a GM or an officer
		-- since we can't be an officer unless we get the the table from the GM, we *should* have
		-- a recent copy. However, there might be a conflict if the GM changes the rank structure,
		-- thus the GM needs to use the "Send Updates" button so that we can minimize the problem.
		if self.db.profile.comm.isGuildMaster or self:IsOfficer(myName) or self:IsOfficerAlt(myName) then
			-- build up the table to send
			for i = 1, GuildControlGetNumRanks() do
				t[i] = self.db.profile.dbCache[myNameRealm].ranks[i]
			end
			-- respond only to the person who sent the request
			self:SendComm("MAIN", "WHISPER", sender, "RGRT", t)
		end
		-- check if the person is already in our user table. If not, add them with 'unknown' version
		if not guilderUsers[sender] then guilderUsers[sender] = "Unknown" end
	end
	-- Response: Get Rank Table - someone sent us a rank table
	if command == "RGRT" then
		-- check if the message contains a table, otherwise ignore it
		if n ~= 1 then
			--[===[@debug@
			if self.db.profile.showDebug then
				self:ShowMessage("RGRT had no parameters!")
			end
			--@end-debug@]===]
			return
		end
		-- get the first parameter
		local t = select(1, ...)
		-- make sure that it is a table
		if type(t) ~= "table" then
			--[===[@debug@
			if self.db.profile.showDebug then
				self:ShowMessage("RGRT was expecting a table!")
			end
			--@end-debug@]===]
			return
		end
		-- check if the ranks we received differ from what we have already
		if self.db.profile.dbCache[myNameRealm].ranks ~= t then
			-- update the ranks table
			for i = 1, GuildControlGetNumRanks() do
				self.db.profile.dbCache[myNameRealm].ranks[i] = t[i] or "3-Member"
			end
			-- make sure the ranks are recreated in UpdateConfigOptions()
			configUpdated = false
			-- refresh the window
			LibStub("AceConfigRegistry-3.0"):NotifyChange("Guilder")
		end
		-- check if the person is already in our user table. If not, add them with 'unknown' version
		if not guilderUsers[sender] then guilderUsers[sender] = "Unknown" end
	end
	-- Update: Get Rank Table - GM notified us that ranks have changed
	if command == "UGRT" then
		self:ShowMessage(L["The GM has changed the ranks. Requesting an update!"])
		self:SendComm("MAIN", "WHISPER", sender, "QGRT")
	end
end

-- general functions --------------------------------------------------------

--- Receives a string and converts it into a table of arguments, using "separator" as the separator character
-- @param message the input string
-- @param separator the separator character
-- @usage Guilder:GetArgs(arg1, " ")
-- @returns table of arguments
function Guilder:GetArgs(message, separator)
	message = message or ""
	local args = {}
	local i = 0
	-- search for seperators in the string and return the separated data
	for value in string.gmatch(message, "[^"..separator.."]+") do
		i = i + 1
		args[i] = string.lower(value)
	end
	return args
end

--- Is ostensibly a wrapper to the normal SendChatMessage function with the following changes:
--     1. Language is set to nil.
--     2. If distribution = "SELF", output is routed to Guilder:ShowMessage() instead.
--     3. If distribution = "NULL", output is muted.
-- The intention for the existence of the NULL option for the channel is to allow for functions to pass a parameter that can be switched between a normal channel and null to turn debug mode features on/off.
-- @param msg the text to send to the recipient
-- @param distribution the name of the channel on which to send it
-- @param target person you want to send the whisper to
-- @usage Guilder:SendMessage("Test", "SELF", "Player")
-- @returns nil
function Guilder:SendMessage(msg, distribution, target)
	--[===[@debug@
	if DEBUG then
		expect(msg, 'typeof', 'string')
		expect(distribution, 'typeof', 'string')
		expect(target, 'typeof', 'nil;string')
	end
	--@end-debug@]===]
	-- check the target
	if not target or target == " " or target == "" then
		target = nil
	end
	-- can't send a whisper if there is no-one to receive it
	if distribution == "WHISPER" and not target then
		--[===[@debug@
		if self.db.profile.showDebug then
			self:ShowMessage("SendMessage Error! Whisper message sent with no recipient")
		end
		--@end-debug@]===]
		return
	end
	-- send chat output to 'chat' or to intended channel if not 'NULL'.
	if distribution == "SELF" then
		self:ShowMessage(msg)
	elseif distribution ~= "NULL" then
		-- if it is a whisper, prepend the addon name to the message
		if distribution == "WHISPER" then msg = "["..L["Guilder"].."] "..msg end
		SendChatMessage(msg, distribution, nil, target)
	end
end

--- Is a wrapper to DEFAULT_CHAT_FRAME:AddMessage()
-- It adds the name of the addon (in colour) to the beggining of the message
-- @param msg the message to show in the chat frame
-- @usage Guilder:ShowMessage("some message")
-- @returns nil
function Guilder:ShowMessage(msg)
	if not msg then return end
	DEFAULT_CHAT_FRAME:AddMessage("|cff00c0ff"..L["Guilder"].."|r - "..msg)
end

-- player functions --------------------------------------------------------

--- Attempts to execute the supplied argument as though it were a command entered by the user in the chat window
-- @param arg the text the user would have typed
-- @usage Guilder:ExecuteChatCommand("/silly")
-- @returns nil
function Guilder:ExecuteChatCommand(arg)
	-- make sure DEFAULT_CHAT_FRAME exists
	if (not DEFAULT_CHAT_FRAME) then return end
	-- enter the text into it
	DEFAULT_CHAT_FRAME.editBox:SetText(arg);
	-- execute the command
	ChatEdit_SendText(DEFAULT_CHAT_FRAME.editBox);
end

--- Attempts to find the officer's name in the notes or officer's notes
-- @param mainName name of the main character
-- @usage Guilder:FindOfficerAlt("Player")
-- @returns "AltName (MainName)" if found, otherwise returns nil
function Guilder:FindOfficerAlt(mainName)
	-- check if mainName is an officer and is currently offline
	if self:IsOfficer(mainName) and (not LibGuild:IsMemberOnline(mainName)) then
		local Note, oNote
		-- go through the guild roster and see if we can find 'mainName' in the notes or officer's notes
		for name in LibGuild:GetIterator("NAME", true) do
			-- get the note
			Note = LibGuild:GetNote(name)
			-- get the officer's note
			oNote = LibGuild:GetOfficerNote(name)
			-- if there is an officer note
			if oNote ~= nil then
				-- if name is online, is an officer alt, and has 'mainName' in the officer's note
				if (LibGuild:IsMemberOnline(name) and self:IsOfficerAlt(name) and string.match(oNote:upper(), mainName:upper())) then
					return mainName.." ("..name..")"
				end
			-- if there is a note
			elseif Note ~= nil then
				-- if name is online, is an officer alt, and has 'mainName' in the note
				if (LibGuild:IsMemberOnline(name) and self:IsOfficerAlt(name) and string.match(Note:upper(), mainName:upper())) then
					return mainName.." ("..name..")"
				end
			end
		end
	end
end

--- Tries returns a coloured string by player class. If the class is unknown, the name is returned in gray.
-- @param name name of the character
-- @usage Guilder:GetClassColorString("Player")
-- @returns the name of the character in the class color
function Guilder:GetClassColorString(name)
	if name == nil then return end
	local _, class = UnitClass(name)
	if not class then return '|cffcccccc'..name.."|r" end
	local c = RAID_CLASS_COLORS[class]
	return "|cff"..("%02X%02X%02X"):format(c.r * 255, c.g * 255, c.b * 255)..name.."|r"
end

--- Looks for characters holding the Officer rank and then simply performs a count to determine how many such characters there
-- @usage Guilder:GetNumberOfGuildOfficers()
-- @returns the number of officers
function Guilder:GetNumberOfGuildOfficers()
	local numOfficers = 0
	-- if we are in a guild
	if IsInGuild() then
		-- go through the guild table
		for name in LibGuild:GetIterator("NAME", true) do
			-- and increase the count if 'name' is an officer
			if self:IsOfficer(name) then numOfficers = numOfficers + 1 end
		end
	end
	return numOfficers
end

--- Looks for characters holding the Officer rank and then simply performs a count to determine how many are online at the moment
-- @usage Guilder:GetNumberOfGuildOfficersOnline()
-- @returns the number of officers
function Guilder:GetNumberOfGuildOfficersOnline()
	local numOfficers = 0
	-- if we are in a guild
	if IsInGuild() then
		-- go through the guild table
		for name in LibGuild:GetIterator("NAME", true) do
			-- and increase the count if 'name' is an officer or officer alt and is currently online
			if (self:IsOfficer(name) or self:IsOfficerAlt(name)) and LibGuild:IsMemberOnline(name) then numOfficers = numOfficers + 1 end
		end
	end
	return numOfficers
end

--- Checks whether a character holds the rank of alt
-- @param name name of the character
-- @usage Guilder:IsAlt("Player")
-- @returns true if the character is an alt, otherwise returns false
function Guilder:IsAlt(name)
	name = name or myName
	return self.db.profile.dbCache[myNameRealm].ranks[LibGuild:GetRankIndex(name) + 1] == "4-Alt"
end

--- Checks whether a character holds the ranks of officer
-- @param name name of the character
-- @usage Guilder:IsOfficer("Player")
-- @returns true if the character is an officer, otherwise returns false
function Guilder:IsOfficer(name)
	name = name or myName
	return self.db.profile.dbCache[myNameRealm].ranks[LibGuild:GetRankIndex(name) + 1] == "1-Officer"
end

--- Checks whether a character holds the rank of officer's alt
-- @param name name of the character
-- @usage Guilder:IsOfficerAlt("Player")
-- @returns true if the character is an officer alt, otherwise returns false
function Guilder:IsOfficerAlt(name)
	name = name or myName
	return self.db.profile.dbCache[myNameRealm].ranks[LibGuild:GetRankIndex(name) + 1] == "2-OffAlt"
end

--- Checks to see if we are currently in a battleground
-- @usage if Guilder:IsInBattleground() then
--     print("You are in a battleground")
-- end
-- @returns true if we are in a battleground, otherwise returns false
function Guilder:IsInBattleground()
	-- quicker check
	return select(2, IsInInstance()) == "pvp"
	--[[
	local status
	-- iterate through all of our battle ground queues
	for i = 1, MAX_BATTLEFIELD_QUEUES do
		-- return true if anyone of them is active
		-- status, mapName, instanceID, minlevel, maxlevel, teamSize
		status, _, _, _, _, _ = GetBattlefieldStatus(i)
		if (status == "active") then
			return true
		end	
	end
	return false
	]]
end

--- Checks to see if we are currently in combat
-- @usage if Guilder:IsInCombat() then
--     print("You are in combat")
-- end
-- @returns true if we are in combat, otherwise returns false
function Guilder:IsInCombat()
	return Guilder.in_combat
end

--- Checks whether the named character is on our friends list
-- @param name name of the character
-- @usage if Guilder:IsInFriends("Player") then
--     print("Player is in your friends list")
-- end
-- @returns true if the character is in the friends list, otherwise returns false
function Guilder:IsInFriends(name)
	local numFriends = GetNumFriends()
	for i = 1, numFriends do
		-- cname, level, class, zone, online, status, note
		local cname, _, _, _, online, _, _ = GetFriendInfo(i)
		if online and cname == name then
			return true
		end
	end
	return false
end

--- Checks to see if we are currently in a party
-- @usage if Guilder:IsInParty() then
--     print("You are in a party")
-- end
-- @returns true if we are in a party, otherwise returns false
function Guilder:IsInParty()
	return LibGroup:IsInGroup() and LibGroup:GetGroupType() == "PARTY"
end

--- Checks to see if we are currently in a raid
-- @usage if Guilder:IsInRaid() then
--     print("You are in a raid")
-- end
-- @returns true if we are in a raid, otherwise returns false
function Guilder:IsInRaid()
	return LibGroup:IsInGroup() and LibGroup:GetGroupType() == "RAID"
end

--- Returns the list of players in our current group.
-- First it attempts to get raid/party data from LibGroup.If we aren't in a group, only our name is added to the list.
-- Note: Will only store our name in Guilder.PlayersInGroup if we are in a battleground.
-- @usage t = Guilder:PlayersInGroup()
-- @returns a table of players in the group
function Guilder:PlayersInGroup()
	local t = {}
	if LibGroup:IsInGroup() then
		for k, _ in LibGroup:Iterate(true) do
			t[k] = "|cff"..LibGroup:GetClassHexColor(k)..k.."|r"
		end
	elseif myName then
		t[myName] = myColorName
	end
	return t
end

--- Returns the list of players in our current guild.
-- @param includeOffline if set to true, will include offline guild members as well
-- @usage t = Guilder:PlayersInGuild()
-- @returns a table of players in the guild
function Guilder:PlayersInGuild(includeOffline)
	local t = {}
	if IsInGuild() then
		for k, _ in LibGuild:GetIterator("NAME", includeOffline) do
			t[k] = "|cff"..LibGuild:GetClassHexColor(k)..k.."|r"
		end
	elseif myName then
		t[myName] = myColorName
	end
	return t
end

--- Returns the list of players in our current group or else guild.
-- Note: Will not add the raid group if we are in a battleground.
-- @usage t = Guilder:PlayersInGroupOrGuild()
-- @returns a table of players in the group or guild
function Guilder:PlayersInGroupOrGuild()
	local t = {}
	if LibGroup:IsInGroup() then
		for k, _ in LibGroup:Iterate(true) do
			t[k] = "|cff"..LibGroup:GetClassHexColor(k)..k.."|r"
		end
		return t
	end
	-- create guild list
	if IsInGuild() then
		for k, _ in LibGuild:GetIterator("NAME") do
			t[k] = "|cff"..LibGuild:GetClassHexColor(k)..k.."|r"
		end
	elseif myName then
		t[myName] = myColorName
	end
	return t
end

--- Returns the list of players in our current group and guild.
-- Note: Will not add the raid group if we are in a battleground.
-- @usage t = Guilder:PlayersInGroupAndGuild()
-- @returns a table of players in the group and guild
function Guilder:PlayersInGroupAndGuild()
	local t = {}
	if LibGroup:IsInGroup() then
		for k, _ in LibGroup:Iterate(true) do
			t[k] = "|cff"..LibGroup:GetClassHexColor(k)..k.."|r"
		end
	end
	-- create guild list
	if IsInGuild() then
		for k, _ in LibGuild:GetIterator("NAME") do
			t[k] = "|cff"..LibGuild:GetClassHexColor(k)..k.."|r"
		end
	else
		if myName then t[myName] = myColorName end
	end
	return t
end

--- Queries the guilderUsers table and returns the version number or nil
-- @param player name of the player to check for
-- @usage Guilder:GetUserVersion(UnitName("Player"))
-- @returns version string or nil
function Guilder:GetUserVersion(player)
	return guilderUsers[player]
end

function Guilder:GetUserBossMods(player)
	return bossMods[player]
end

function Guilder:GetBigWigs(player)
	if bossMods[player] and bossMods[player].BW then
		return bossMods[player].BW.ver, bossMods[player].BW.rev
	end
end

function Guilder:GetDBM(player)
	if bossMods[player] and bossMods[player].DBM then
		return bossMods[player].DBM.ver, bossMods[player].DBM.rev
	end
end

function Guilder:GetDXE(player)
	if bossMods[player] and bossMods[player].DXE then
		return bossMods[player].DXE.ver, bossMods[player].DXE.rev
	end
end
