--- Guilder: Responder Module
-- This module allows the user to have a response message sent when a whisper is received
-- @class file
-- @name Modules/responder.lua
-- @release 2.0.x

--[[
   Copyright 2009 Christo Kotze (onaforeignshore)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
]]

if select(6, GetAddOnInfo("Guilder_" .. (debugstack():match("[o%.][d%.][u%.]les\\(.-)\\") or ""))) ~= "MISSING" then return end

local Guilder = _G.Guilder
if not Guilder then
  error("Guilder_Responder requires Guilder")
end
local L = Guilder.L

local DEBUG = Guilder.DEBUG
local expect = Guilder.expect

-- default settings for the addon -------------------------------------------

local DATABASE_DEFAULTS = {
	["message"] = L["Thank you for your message. Unfortunately, the recipient is unavailable to respond at this time but your message has been stored for later review."],
	["officer"] = false,
	["guild"] = false,
	["party"] = false,
	["raid"] = false,
	["friend"] = false
}

-- declare the module -------------------------------------------------------

local Guilder_Responder = Guilder:NewModule("Responder", "AceEvent-3.0", "AceHook-3.0")
Guilder_Responder:SetName(L["Busy Responder"])
Guilder_Responder:SetDescription(L["Responds to whispers when you're not up to it."])
Guilder_Responder:SetDefaults(DATABASE_DEFAULTS)
if DEBUG then
	Guilder_Responder:SetIcon([[Interface\AddOns\Guilder\Modules\Responder\Responder]])
else
	Guilder_Responder:SetIcon([[Interface\AddOns\Guilder_Responder\Responder]])
end

-- variables ----------------------------------------------------------------

local addonName = "["..L["Guilder"].."]"
local busyResponder = addonName.." "..L["Busy Responder"]

-- Standard Module Methods --------------------------------------------------

-- Is called when the module is loaded
-- @usage internal use only
function Guilder_Responder:OnModuleLoaded(module)
	if not self.db.profile.enabled then return end
	local id = module.id
end

-- Is called when the module is enabled
-- @usage internal use only
function Guilder_Responder:OnEnable()
	-- Catch whispers and filter what should be shown
	-- TODO: stop WIM from showing everyting
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("CHAT_MSG_WHISPER_INFORM")
end

-- Is called when the module is disabled
-- @usage internal use only
function Guilder_Responder:OnDisable()
	-- Called when the addon is disabled
	self:UnregisterAllEvents()
end

-- Is called when a whisper is received
-- @usage internal use only
function Guilder_Responder:CHAT_MSG_WHISPER(...)
	local msg, _ = strsplit(" ", arg1, 2)
	if self.db.profile.enabled then
		if msg:lower() ~= addonName:lower() then self:SendBusyResponse(arg2, arg1) end
		return false, ...
	end
	-- return true, ...
end

-- Is called when a whisper is sent
-- @usage internal use only
function Guilder_Responder:CHAT_MSG_WHISPER_INFORM(...)
	local msg, _ = strsplit(" ", arg1, 2)
	-- ignore messages from our addon
	if msg:lower() == addonName:lower() then
		arg1 = ""
		return true, ...
	end
	-- return false, ...
end

--[[
-- Is called when the user opens the configuration screen
-- @usage internal use only
function Guilder_Responder:UpdateConfigOptions()
end

-- Is called when the module receives a message
-- @usage internal use only
function Guilder_Responder:CommHandler(distribution, sender, command, ...)
end
]]

-- General functions --------------------------------------------------------

-- Sends a response to the sender of a whispered message if necessary
-- User can choose to respond to any of the following:
--     o  someone in our guild.
--     o  someone an officer of our guild.
--     o  someone in our party.
--     o  someone in our raid.
--     o  someone on our friends-list.
-- @param sender name of the person who whispered us
-- @param msg the original message received
-- @usage internal use only
-- @returns nil
function Guilder_Responder:SendBusyResponse(sender, msg)
	-- check if sender is valid
	if not sender or sender == "" then
		return
	end
	-- perform the officer-member exclusion check
	if not self.db.profile.officer and LibGuild:HasMember(sender) and Guilder:IsOfficer(sender) then
		return
	end
	-- perform the guild-member exclusion check
	if not self.db.profile.guild and LibGuild:HasMember(sender) then
		return
	end
	-- perform the raid-member exclusion check
	if not self.db.profile.raid and UnitInRaid(sender) then
		return
	end
	-- perform the party-member exclusion check
	if not self.db.profile.party and UnitInParty(sender) then
		return
	end
	-- perform the friends-list exclusion check
	if not self.db.profile.friend and Guilder:UnitInFriends(sender) then
		return
	end
	-- send the response message
	Guilder:SendMessage(L["Busy Responder"].." - "..self.db.profile.message, "WHISPER", sender)
end
