--- Guilder: Version Checker Module
-- This module allows the user to query the versions of raiding addons installed on another member of the group's computer.
-- @class file
-- @name Modules/Version.lua
-- @release 2.0.x

--[[
   Copyright 2009 Christo Kotze (onaforeignshore)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
]]

if select(6, GetAddOnInfo("Guilder_" .. (debugstack():match("[o%.][d%.][u%.]les\\(.-)\\") or ""))) ~= "MISSING" then return end

local Guilder = _G.Guilder
if not Guilder then
  error("Guilder_Version requires Guilder")
end
local L = Guilder.L

local DEBUG = Guilder.DEBUG
local expect = Guilder.expect

-- default settings for the addon -------------------------------------------

local DATABASE_DEFAULTS = {
	channel = "",
	player = "",
	queryGuilder = true,
	queryDBM = true,
	queryOmen = true,
	queryRecount = true,
	includeRevision = false
}

-- declare the module -------------------------------------------------------

local Guilder_Version = Guilder:NewModule("Version", "AceEvent-3.0", "AceHook-3.0")
Guilder_Version:SetName(L["Version Checker"])
Guilder_Version:SetDescription(format(L["Check addon versions."]))
Guilder_Version:SetDefaults(DATABASE_DEFAULTS)
if DEBUG then
	Guilder_Version:SetIcon([[Interface\AddOns\Guilder\Modules\Version\Version]])
else
	Guilder_Version:SetIcon([[Interface\AddOns\Guilder_Version\Version]])
end

-- variables ----------------------------------------------------------------

-- Standard Module Methods --------------------------------------------------

-- Is called when the module is loaded
-- @usage internal use only
function Guilder_Version:OnModuleLoaded(module)
	if not self.db.profile.enabled then return end
	local id = module.id
end

-- Is called when the module is enabled
-- @usage internal use only
function Guilder_Version:OnEnable()
end

-- Is called when the module is disabled
-- @usage internal use only
function Guilder_Version:OnDisable()
end

--[[
-- Is called when the user opens the configuration screen
-- @usage internal use only
function Guilder_Version:UpdateConfigOptions()
end
]]


-- Is called when the module receives a message
-- @usage internal use only
function Guilder_Version:CommHandler(distribution, sender, command, ...)
	-- if module is disabled, do nothing
	if not self.db.profile.enabled then
		return
	end
	-- only handle comms if they are not from yourself
	-- if sender == myName then return end
	local t = {}
	local n = select('#', ...)
	-- Query: Get Version Check
	if command == "QGVC" then
		if self.db.profile.queryBigwigs then
			t["BigWigs"] = ((BigwigsLoader and L["Installed"]) or L["Not Installed!"])
		end
		if self.db.profile.queryDBM then
			t["DBM"] = ((DBM and DBM.Version) or L["Not Installed!"])
		end
		if self.db.profile.queryDBM then
			t["DXE"] = ((DXE and DXE.version) or L["Not Installed!"])
		end
		if self.db.profile.queryOmen then
			t["Omen"] = ((Omen and Omen.version) or L["Not Installed!"])
		end
		if self.db.profile.queryRecount then
			t["Recount"] = ((Recount and Recount.Version) or L["Not Installed!"])
		end
		Guilder:SendComm(self.id, "WHISPER", sender, "RGVC", t)
	end
	-- Response: Get Version Check
	if command == "RGVC" then
		-- check if the message contains a table, otherwise ignore it
		if n ~= 1 then
			if self.db.profile.showDebug then
				self:ShowMessage("RGVC had no parameters!")
			end
			return
		end
		-- get the first parameter
		local t = select(1, ...)
		-- make sure that it is a table
		if type(t) ~= "table" then
			if self.db.profile.showDebug then
				self:ShowMessage("RGVC was expecting a table!")
			end
			return
		end
		local msg = Guilder:GetClassColorString(sender).." - "
		if self.db.profile.queryBigwigs then
			msg = msg.." "..L["BigWigs"]..": "..(t["BigWigs"] or L["Not installed"])
		end
		if self.db.profile.queryDBM then
			msg = msg.." "..L["DBM"]..": "..(t["DBM"] or L["Not installed"])
		end
		if self.db.profile.queryDXE then
			msg = msg.." "..L["DXE"]..": "..(t["DXE"] or L["Not installed or not enabled"])
		end
		if self.db.profile.queryOmen then
			msg = msg.." "..L["Omen"]..": "..(t["Omen"] or L["Not installed"])
		end
		if self.db.profile.queryRecount then
			msg = msg.." "..L["Recount"]..": "..(t["Recount"] or L["Not installed"])
		end
		Guilder:ShowMessage(msg)
	end
end

-- General functions --------------------------------------------------------

-- Sends a request to all players to get the versions of raid addons installed
function Guilder_Version:QueryPlayerVersions()
	if not self.db.profile.enabled then
		Guilder:ShowMessage(L["%s is disabled"]:format("|cff00c0ff"..L["Version Checker"].."|r"))
		return
	end
	if not (self.db.profile.queryDBM or self.db.profile.queryOmen or self.db.profile.queryRecount) then
		Guilder:ShowMessage(L["%s - You have not chosen any addons to check for!"]:format("|cff00c0ff"..L["Version Checker"].."|r"))
		return
	end
	if self.db.profile.channel == "GROUP" then
		if not (LibGroup:IsInGroup()) then
			Guilder:ShowMessage(L["%s - You are not in a group!"]:format("|cff00c0ff"..L["Version Checker"].."|r"))
			return
		end
		Guilder:ShowMessage(L["%s is querying the group"]:format("|cff00c0ff"..L["Version Checker"].."|r"))
		Guilder:SendComm(self.id, LibGroup:GetGroupType(), myName, "QGVC")
	else
		if self.db.profile.channel == "WHISPER" and self.db.profile.player == "" then
			Guilder:ShowMessage("|cff00c0ff"..L["Version Checker"].."|r - "..L["You have to choose a player to query!"])
			return
		end
		Guilder:SendComm(self.id, self.db.profile.channel, self.db.profile.player, "QGVC")
	end
end

function Guilder_Version:ShowGroupVersions()
	if not self.db.profile.enabled then
		Guilder:ShowMessage(L["%s is disabled"]:format("|cff00c0ff"..L["Version Checker"].."|r"))
		return
	end
	if not (LibGroup:IsInGroup()) then
		Guilder:ShowMessage(L["%s - You are not in a group!"]:format("|cff00c0ff"..L["Version Checker"].."|r"))
		return
	end
	local msg
	local found = true
	Guilder:ShowMessage(L["%s - Group Boss Mod versions"]:format("|cff00c0ff"..L["Version Checker"].."|r"))
	for k, _ in LibGroup:Iterate(true) do
		msg = "|cff"..LibGroup:GetClassHexColor(k)..k.."|r - "
		local ver, rev = Guilder:GetBigWigs(k)
		if ver then
			msg = msg..L["BigWigs"]..": "..ver.." "
			if self.db.profile.includeRevision and rev then
				msg = msg.." ("..rev..")"
			end
			found = false
		end
		ver, rev = Guilder:GetDBM(k)
		if ver then
			msg = msg..L["DBM"]..": "..ver.." "
			if self.db.profile.includeRevision and rev then
				msg = msg.." ("..rev..")"
			end
			found = false
		end
		ver, rev = Guilder:GetDXE(k)
		if ver then
			msg = msg..L["DXE"]..": "..ver.." "
			if self.db.profile.includeRevision and rev then
				msg = msg.." ("..rev..")"
			found = false
			end
		end
		if found then msg = msg.." "..L["None"] end
		Guilder:ShowMessage(msg)
	end
end